/*	This program reads an input file of x and y data and smooths the data
	using an integration method.

	The input file is requested along with the gate width.

	The input file must be in the current directory

	The output file is given the extension .SMO and placed in current dir

	The input data should be separated with spaces or commas.(changable)

	The problem with this version is that the data within a gate width is all
	given the same value.  an improved version will be done that makes the
	points between a gate flow together with a straight line of the
	appropriate slope.

*/

#include <alloc.h>  		/* malloc,free */
#include <conio.h>		/* for clrscr */
#include <stdio.h>		/* fopen,fclose */
#include <io.h>			/* access */
#include <string.h>		/* strtok,strupr */
#include <float.h>		/* scanf,printf float format */
#include <stdlib.h>		/* atoi */
#include <math.h>			/* need _matherror to handle file input for some */
								/*   strange reason ??								    */


int openfile(void);
void readdata(void);
void smoothdata(void);
void toutfini(void);
void writedata(void);

FILE *infile,*outfile;						/* file pointers for input/output */
char *inname,*outname;						/* file names for input/output */

char name[13];


struct datatype {								/* declare and define structure */
	float x;										/*  named dataset of type datatype */
	float y;										/*  next points to next list item */
	struct datatype *next;
} dataset;


struct datatype *first,*last,*prev;		/* pointers to fisrt and last data */

int items,namelen;


/***************************************************************************
**************************** MAIN ******************************************
***************************************************************************/

main() {
	if(openfile()){								/* opens input data file */
		readdata();									/* reads data into linked list */
		smoothdata();								/* uses integration to remove noise */
		writedata();								/* writes smoothed data to file */
		toutfini();									/* tidies up things and exits */
	}
}



/***************************************************************************
**************************** OpenFile **************************************
***************************************************************************/
int openfile() {
/*	clrscr();*/
	inname = (char *) malloc(sizeof(char));   /* must allocate mem first */
	printf("Enter input file name: ");
	scanf("%s",inname);

	if (access(inname,00)!=0) {
		printf("Cant find file!..\n");
		return(0);
	}
	else {
		infile = fopen(inname,"rt");		/* open file read only, text format */
	}
	outname = (char *) malloc(sizeof(char));		/* must alloc mem first */
	outname = strtok(inname,".");
	namelen = strlen(outname);
	return(1);
}



/***************************************************************************
**************************** ReadData **************************************
***************************************************************************/
void readdata() {
	static struct datatype *current,*prev;
	static char *xchar,*ychar;
	static char line[80];

	xchar = (char *) malloc(sizeof(char));   	/* must allocate mem first */
	ychar = (char *) malloc(sizeof(char));   	/* must allocate mem first */

	first = NULL;										/* initialize list markers */
	last = NULL;
	current = NULL;

	while(fgets(line,80,infile)!=NULL){
		prev = current;                        /* reset previous identifier */

		current = (struct datatype *)          /* allocate mem for list */
					malloc(sizeof(dataset));		/*								 */
		if (!current) {								/* bye bye if no more memory */
			printf("out of memory!\n");
			return;
		}

		xchar = strtok(line,", ");				/* parse x char data from string */
		ychar = strtok(NULL,", ");				/* parse y char data from string */

		current->x = atof(xchar);	 			/* convert x to numerical data 	*/
		current->y = atof(ychar); 				/* convert y to numerical data 	*/


		if(first == NULL)                   /* if first item in list			*/
			first = current;						/*   set first item place marker */
		else {                  				/* if not first item 				*/
			last = current;						/*   make current item the last 	*/
			prev->next = current;				/*   update pointer to next item */
		}                                   /* end else */

		items ++;									/* increment data items  counter */
	}     											/* end while 							*/

}  													/* end of readdata */


/***************************************************************************
**************************** SmoothData ************************************
***************************************************************************/
void smoothdata() {
	static int cnt,GW;
	float avesum;
	static struct datatype *current,*prev,*temp;


	cputs("Enter Gate Width: ");
	scanf("%d",&GW);
	GW += 1;												/* Corrects for input */

	prev = first;										/* Set first half of gate */
	current = first;									/* ititialize second half */
	while(current<last) {							/* <= may not be what I want */

		for(cnt=1;(cnt<=GW-1&&current<last);cnt++){  				/* jump to second half of gate */
			current = current->next;
		}


		temp = prev;
		avesum = prev->y;
		for(cnt=1;(cnt<=GW-1&&prev<last);cnt++){	/* loop to calc the ave    */
			prev = prev->next;							/*   value in the gate     */
			avesum += prev->y;
		}
		avesum = avesum / (float)cnt;


		prev = temp;										/* loop to set all values  */
		for(cnt=1;(cnt<=GW-1&&prev<last);cnt++){	/*   in gate to the average*/
			prev->y = avesum;								/*   value						*/
			prev = prev->next;
		}
	}



/*
current = first;
for(cnt=1;cnt<=items;cnt++) {
 printf("item:%5d  x:%8.3f  y:%8.3f      cur:%p  next:%p\n",
 cnt,current->x,current->y,current,current->next);
 current = current->next;
} */

}


/***************************************************************************
**************************** WriteData *************************************
***************************************************************************/
void writedata() {
	int cnt;


	static struct datatype *current;
/*	outname = "";*/
	strncpy(name,inname,namelen);
	strcat(name,".SMO");
	strupr(name);
	outfile = fopen(name,"wt");

	current = first;
	for(cnt=1;cnt<=items;cnt++) {
		fprintf(outfile,"%f,%-f\n",current->x,current->y);
		current = current->next;
	}
}


/***************************************************************************
**************************** ToutFini **************************************
***************************************************************************/
void toutfini() {

	fclose(infile);
	fclose(outfile);
	printf("\n%d data items evaluated \n",items);
	printf("Output file: %s\n",name);
}




